@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,23 +14,25 @@
 # limitations under the License.
 #
 from collections import OrderedDict
-from distutils import util
 import os
 import re
 from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core import client_options as client_options_lib  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core import client_options as client_options_lib
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport import mtls  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 from google.auth.exceptions import MutualTLSChannelError  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.api_core import operation  # type: ignore
 from google.api_core import operation_async  # type: ignore
@@ -293,6 +295,73 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
         return m.groupdict() if m else {}
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[client_options_lib.ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        if client_options is None:
+            client_options = client_options_lib.ClientOptions()
+        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
+        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
+        if use_client_cert not in ("true", "false"):
+            raise ValueError(
+                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
+            )
+        if use_mtls_endpoint not in ("auto", "never", "always"):
+            raise MutualTLSChannelError(
+                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
+            )
+
+        # Figure out the client cert source to use.
+        client_cert_source = None
+        if use_client_cert == "true":
+            if client_options.client_cert_source:
+                client_cert_source = client_options.client_cert_source
+            elif mtls.has_default_client_cert_source():
+                client_cert_source = mtls.default_client_cert_source()
+
+        # Figure out which api endpoint to use.
+        if client_options.api_endpoint is not None:
+            api_endpoint = client_options.api_endpoint
+        elif use_mtls_endpoint == "always" or (
+            use_mtls_endpoint == "auto" and client_cert_source
+        ):
+            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
+        else:
+            api_endpoint = cls.DEFAULT_ENDPOINT
+
+        return api_endpoint, client_cert_source
+
     def __init__(
         self,
         *,
@@ -343,50 +412,22 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         if client_options is None:
             client_options = client_options_lib.ClientOptions()
 
-        # Create SSL credentials for mutual TLS if needed.
-        use_client_cert = bool(
-            util.strtobool(os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"))
+        api_endpoint, client_cert_source_func = self.get_mtls_endpoint_and_cert_source(
+            client_options
         )
 
-        client_cert_source_func = None
-        is_mtls = False
-        if use_client_cert:
-            if client_options.client_cert_source:
-                is_mtls = True
-                client_cert_source_func = client_options.client_cert_source
-            else:
-                is_mtls = mtls.has_default_client_cert_source()
-                if is_mtls:
-                    client_cert_source_func = mtls.default_client_cert_source()
-                else:
-                    client_cert_source_func = None
-
-        # Figure out which api endpoint to use.
-        if client_options.api_endpoint is not None:
-            api_endpoint = client_options.api_endpoint
-        else:
-            use_mtls_env = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
-            if use_mtls_env == "never":
-                api_endpoint = self.DEFAULT_ENDPOINT
-            elif use_mtls_env == "always":
-                api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-            elif use_mtls_env == "auto":
-                if is_mtls:
-                    api_endpoint = self.DEFAULT_MTLS_ENDPOINT
-                else:
-                    api_endpoint = self.DEFAULT_ENDPOINT
-            else:
-                raise MutualTLSChannelError(
-                    "Unsupported GOOGLE_API_USE_MTLS_ENDPOINT value. Accepted "
-                    "values: never, auto, always"
-                )
+        api_key_value = getattr(client_options, "api_key", None)
+        if api_key_value and credentials:
+            raise ValueError(
+                "client_options.api_key and credentials are mutually exclusive"
+            )
 
         # Save or instantiate the transport.
         # Ordinarily, we provide the transport, but allowing a custom transport
         # instance provides an extensibility point for unusual situations.
         if isinstance(transport, AssetServiceTransport):
             # transport is a AssetServiceTransport instance.
-            if credentials or client_options.credentials_file:
+            if credentials or client_options.credentials_file or api_key_value:
                 raise ValueError(
                     "When providing a transport instance, "
                     "provide its credentials directly."
@@ -398,6 +439,15 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
                 )
             self._transport = transport
         else:
+            import google.auth._default  # type: ignore
+
+            if api_key_value and hasattr(
+                google.auth._default, "get_api_key_credentials"
+            ):
+                credentials = google.auth._default.get_api_key_credentials(
+                    api_key_value
+                )
+
             Transport = type(self).get_transport_class(transport)
             self._transport = Transport(
                 credentials=credentials,
@@ -432,6 +482,34 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         the export operation result. For regular-size resource parent,
         the export operation usually finishes within 5 minutes.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_export_assets():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                output_config = asset_v1.OutputConfig()
+                output_config.gcs_destination.uri = "uri_value"
+
+                request = asset_v1.ExportAssetsRequest(
+                    parent="parent_value",
+                    output_config=output_config,
+                )
+
+                # Make the request
+                operation = client.export_assets(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.ExportAssetsRequest, dict]):
                 The request object. Export asset request.
@@ -496,6 +574,27 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         r"""Lists assets with time and resource types and returns
         paged results in response.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_list_assets():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.ListAssetsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                page_result = client.list_assets(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.ListAssetsRequest, dict]):
                 The request object. ListAssets request.
@@ -524,7 +623,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -582,6 +681,26 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         specified asset does not exist, this API returns an
         INVALID_ARGUMENT error.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_batch_get_assets_history():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.BatchGetAssetsHistoryRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.batch_get_assets_history(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.BatchGetAssetsHistoryRequest, dict]):
                 The request object. Batch get assets history request.
@@ -632,6 +751,31 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         project/folder/organization to listen to its asset
         updates.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_create_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                feed = asset_v1.Feed()
+                feed.name = "name_value"
+
+                request = asset_v1.CreateFeedRequest(
+                    parent="parent_value",
+                    feed_id="feed_id_value",
+                    feed=feed,
+                )
+
+                # Make the request
+                response = client.create_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.CreateFeedRequest, dict]):
                 The request object. Create asset feed request.
@@ -667,7 +811,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -714,6 +858,25 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
     ) -> asset_service.Feed:
         r"""Gets details about an asset feed.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_get_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.GetFeedRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.GetFeedRequest, dict]):
                 The request object. Get asset feed request.
@@ -744,7 +907,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -792,6 +955,26 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         r"""Lists all asset feeds in a parent
         project/folder/organization.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_list_feeds():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.ListFeedsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.list_feeds(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.ListFeedsRequest, dict]):
                 The request object. List asset feeds request.
@@ -817,7 +1000,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -864,6 +1047,28 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
     ) -> asset_service.Feed:
         r"""Updates an asset feed configuration.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_update_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                feed = asset_v1.Feed()
+                feed.name = "name_value"
+
+                request = asset_v1.UpdateFeedRequest(
+                    feed=feed,
+                )
+
+                # Make the request
+                response = client.update_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.UpdateFeedRequest, dict]):
                 The request object. Update asset feed request.
@@ -895,7 +1100,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([feed])
         if request is not None and has_flattened_params:
@@ -944,6 +1149,22 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
     ) -> None:
         r"""Deletes an asset feed.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_delete_feed():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.DeleteFeedRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_feed(request=request)
+
         Args:
             request (Union[google.cloud.asset_v1.types.DeleteFeedRequest, dict]):
                 The request object.
@@ -963,7 +1184,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -1014,6 +1235,27 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         the ``cloudasset.assets.searchAllResources`` permission on the
         desired scope, otherwise the request will be rejected.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_search_all_resources():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.SearchAllResourcesRequest(
+                    scope="scope_value",
+                )
+
+                # Make the request
+                page_result = client.search_all_resources(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.SearchAllResourcesRequest, dict]):
                 The request object. Search all resources request.
@@ -1125,7 +1367,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([scope, query, asset_types])
         if request is not None and has_flattened_params:
@@ -1186,6 +1428,27 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         ``cloudasset.assets.searchAllIamPolicies`` permission on the
         desired scope, otherwise the request will be rejected.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_search_all_iam_policies():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.SearchAllIamPoliciesRequest(
+                    scope="scope_value",
+                )
+
+                # Make the request
+                page_result = client.search_all_iam_policies(request=request)
+
+                # Handle the response
+                for response in page_result:
+                    print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.SearchAllIamPoliciesRequest, dict]):
                 The request object. Search all IAM policies request.
@@ -1279,7 +1542,7 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([scope, query])
         if request is not None and has_flattened_params:
@@ -1334,6 +1597,29 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         r"""Analyzes IAM policies to answer which identities have
         what accesses on which resources.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_analyze_iam_policy():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                analysis_query = asset_v1.IamPolicyAnalysisQuery()
+                analysis_query.scope = "scope_value"
+
+                request = asset_v1.AnalyzeIamPolicyRequest(
+                    analysis_query=analysis_query,
+                )
+
+                # Make the request
+                response = client.analyze_iam_policy(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.AnalyzeIamPolicyRequest, dict]):
                 The request object. A request message for
@@ -1397,6 +1683,37 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         to poll the operation result. The metadata contains the metadata
         for the long-running operation.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_analyze_iam_policy_longrunning():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                analysis_query = asset_v1.IamPolicyAnalysisQuery()
+                analysis_query.scope = "scope_value"
+
+                output_config = asset_v1.IamPolicyAnalysisOutputConfig()
+                output_config.gcs_destination.uri = "uri_value"
+
+                request = asset_v1.AnalyzeIamPolicyLongrunningRequest(
+                    analysis_query=analysis_query,
+                    output_config=output_config,
+                )
+
+                # Make the request
+                operation = client.analyze_iam_policy_longrunning(request=request)
+
+                print("Waiting for operation to complete...")
+
+                response = operation.result()
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.AnalyzeIamPolicyLongrunningRequest, dict]):
                 The request object. A request message for
@@ -1469,6 +1786,27 @@ class AssetServiceClient(metaclass=AssetServiceClientMeta):
         subject to change before the actual resource migration
         takes place.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1
+
+            def sample_analyze_move():
+                # Create a client
+                client = asset_v1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1.AnalyzeMoveRequest(
+                    resource="resource_value",
+                    destination_parent="destination_parent_value",
+                )
+
+                # Make the request
+                response = client.analyze_move(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1.types.AnalyzeMoveRequest, dict]):
                 The request object. The request message for performing
