@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright 2020 Google LLC
+# Copyright 2022 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -16,17 +16,20 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Sequence, Tuple, Type, Union
+from typing import Dict, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
-from google.api_core.client_options import ClientOptions  # type: ignore
-from google.api_core import exceptions as core_exceptions  # type: ignore
-from google.api_core import gapic_v1  # type: ignore
-from google.api_core import retry as retries  # type: ignore
+from google.api_core.client_options import ClientOptions
+from google.api_core import exceptions as core_exceptions
+from google.api_core import gapic_v1
+from google.api_core import retry as retries
 from google.auth import credentials as ga_credentials  # type: ignore
 from google.oauth2 import service_account  # type: ignore
 
-OptionalRetry = Union[retries.Retry, object]
+try:
+    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault]
+except AttributeError:  # pragma: NO COVER
+    OptionalRetry = Union[retries.Retry, object]  # type: ignore
 
 from google.cloud.asset_v1p2beta1.types import asset_service
 from .transports.base import AssetServiceTransport, DEFAULT_CLIENT_INFO
@@ -98,6 +101,42 @@ class AssetServiceAsyncClient:
 
     from_service_account_json = from_service_account_file
 
+    @classmethod
+    def get_mtls_endpoint_and_cert_source(
+        cls, client_options: Optional[ClientOptions] = None
+    ):
+        """Return the API endpoint and client cert source for mutual TLS.
+
+        The client cert source is determined in the following order:
+        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
+        client cert source is None.
+        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
+        default client cert source exists, use the default one; otherwise the client cert
+        source is None.
+
+        The API endpoint is determined in the following order:
+        (1) if `client_options.api_endpoint` if provided, use the provided one.
+        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
+        default mTLS endpoint; if the environment variabel is "never", use the default API
+        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
+        use the default API endpoint.
+
+        More details can be found at https://google.aip.dev/auth/4114.
+
+        Args:
+            client_options (google.api_core.client_options.ClientOptions): Custom options for the
+                client. Only the `api_endpoint` and `client_cert_source` properties may be used
+                in this method.
+
+        Returns:
+            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
+                client cert source to use.
+
+        Raises:
+            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
+        """
+        return AssetServiceClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore
+
     @property
     def transport(self) -> AssetServiceTransport:
         """Returns the transport used by the client instance.
@@ -171,6 +210,31 @@ class AssetServiceAsyncClient:
         project/folder/organization to listen to its asset
         updates.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1p2beta1
+
+            def sample_create_feed():
+                # Create a client
+                client = asset_v1p2beta1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                feed = asset_v1p2beta1.Feed()
+                feed.name = "name_value"
+
+                request = asset_v1p2beta1.CreateFeedRequest(
+                    parent="parent_value",
+                    feed_id="feed_id_value",
+                    feed=feed,
+                )
+
+                # Make the request
+                response = client.create_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1p2beta1.types.CreateFeedRequest, dict]):
                 The request object. Create asset feed request.
@@ -206,7 +270,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -253,6 +317,25 @@ class AssetServiceAsyncClient:
     ) -> asset_service.Feed:
         r"""Gets details about an asset feed.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1p2beta1
+
+            def sample_get_feed():
+                # Create a client
+                client = asset_v1p2beta1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1p2beta1.GetFeedRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                response = client.get_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1p2beta1.types.GetFeedRequest, dict]):
                 The request object. Get asset feed request.
@@ -283,7 +366,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
@@ -341,6 +424,26 @@ class AssetServiceAsyncClient:
         r"""Lists all asset feeds in a parent
         project/folder/organization.
 
+
+        .. code-block:: python
+
+            from google.cloud import asset_v1p2beta1
+
+            def sample_list_feeds():
+                # Create a client
+                client = asset_v1p2beta1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1p2beta1.ListFeedsRequest(
+                    parent="parent_value",
+                )
+
+                # Make the request
+                response = client.list_feeds(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1p2beta1.types.ListFeedsRequest, dict]):
                 The request object. List asset feeds request.
@@ -366,7 +469,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([parent])
         if request is not None and has_flattened_params:
@@ -423,6 +526,28 @@ class AssetServiceAsyncClient:
     ) -> asset_service.Feed:
         r"""Updates an asset feed configuration.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1p2beta1
+
+            def sample_update_feed():
+                # Create a client
+                client = asset_v1p2beta1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                feed = asset_v1p2beta1.Feed()
+                feed.name = "name_value"
+
+                request = asset_v1p2beta1.UpdateFeedRequest(
+                    feed=feed,
+                )
+
+                # Make the request
+                response = client.update_feed(request=request)
+
+                # Handle the response
+                print(response)
+
         Args:
             request (Union[google.cloud.asset_v1p2beta1.types.UpdateFeedRequest, dict]):
                 The request object. Update asset feed request.
@@ -454,7 +579,7 @@ class AssetServiceAsyncClient:
 
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([feed])
         if request is not None and has_flattened_params:
@@ -503,6 +628,22 @@ class AssetServiceAsyncClient:
     ) -> None:
         r"""Deletes an asset feed.
 
+        .. code-block:: python
+
+            from google.cloud import asset_v1p2beta1
+
+            def sample_delete_feed():
+                # Create a client
+                client = asset_v1p2beta1.AssetServiceClient()
+
+                # Initialize request argument(s)
+                request = asset_v1p2beta1.DeleteFeedRequest(
+                    name="name_value",
+                )
+
+                # Make the request
+                client.delete_feed(request=request)
+
         Args:
             request (Union[google.cloud.asset_v1p2beta1.types.DeleteFeedRequest, dict]):
                 The request object.
@@ -522,7 +663,7 @@ class AssetServiceAsyncClient:
                 sent along with the request as metadata.
         """
         # Create or coerce a protobuf request object.
-        # Sanity check: If we got a request object, we should *not* have
+        # Quick check: If we got a request object, we should *not* have
         # gotten any keyword arguments that map to the request.
         has_flattened_params = any([name])
         if request is not None and has_flattened_params:
